(in-package "GWB")

(defconstant *months*
  '(nil "January" "February" "March" "April" "May" "June" "July"
    "September" "October" "November" "December"))

(defconstant *days-of-week*
  '("Monday" "Tuesday" "Wednesday" "Thursday" "Friday" "Saturday" "Sunday"))

(defun print-universal-time-words (s time)
  (multiple-value-bind
        (second minute hour date month year day-of-week)
      (decode-universal-time time)
    (declare (ignore second))
  (format s "~A the ~@(~:R~) of ~A ~D at ~D:~2,'0D"
          (nth day-of-week *days-of-week*)
          date (nth month *months*) year
          hour minute)))

(defun print-universal-time-compact (s time)
  (multiple-value-bind
        (second minute hour date month year)
      (decode-universal-time time)
    (declare (ignore second))
    (format s "~D.~@R.~4D ~D:~2,'0D"
          date month year
          hour minute)))

#+(or CMU CLISP)
(defun make-temp-file-name ()
  (format nil "/tmp/~A~D"
          (gensym "gwb")
          #+CMU (unix:unix-getpid)
          #+CLISP (system::program-id)))


(defvar *running-processes* '())

(defun delete-file-no-error (filename)
  (ignore-errors
    (delete-file filename)))

(defun check-running-processes ()
  (setf *running-processes*
        (loop for entry in *running-processes*
              for process = (car entry)
              for file = (cdr entry)
              if
                #+CMU
                (member (ext:process-status (car entry))
                        '(:exited :signaled))
                #-CMU t
                do (delete-file-no-error (cdr entry))
               else
                 collect entry)))

(defun kill-running-processes ()
  (dolist (entry *running-processes*)
    #+CMU
    (unix:unix-kill (ext:process-pid (car entry)) :SIGKILL)
    (delete-file-no-error (cdr entry)))
  (setf *running-processes* '()))

(defun display-run (s)
  #-(or CMU CLISP)
  (error 'gwb-simple-error
         :format-control "Not implemented on this platform")
  (let ((filename (make-temp-file-name)))
    (with-open-file (out filename :direction :output)
      (print-strategy-history #'eps-printer
                              s out))
    #+CMU
    (progn
           (let ((p))
             (unwind-protect
                  (progn
                    (setf p
                          (ext:run-program
                           *postscript-viewer-program-name*
                           (list filename)
                           :wait nil
                           :output *error-output*
                           :error *error-output*))
                    (when (null p)
                      (error 'gwb-simple-error
                             :format-control
                             "Couldn't run postscript viewer")))
               (unless p
                 (delete-file-no-error filename)))
             (when p
               (setf *running-processes*
                     (acons p filename *running-processes*)))))
    #+CLISP
    (unwind-protect
         (run-program
          *postscript-viewer-program-name*
          :arguments (list filename))
      (delete-file-no-error filename))))

